import sys
import typing
import freestyle.types
import bpy.types
import mathutils

GenericType = typing.TypeVar("GenericType")


class BackboneStretcherShader:
    ''' Class hierarchy: `freestyle.types.StrokeShader` > `BackboneStretcherShader` [Geometry shader]
    '''

    def __init__(self, amount: float = 2.0):
        ''' Builds a BackboneStretcherShader object.

        :param amount: The stretching amount value.
        :type amount: float
        '''
        pass

    def shade(self, stroke: 'freestyle.types.Stroke'):
        ''' Stretches the stroke at its two extremities and following the respective directions: v(1)v(0) and v(n-1)v(n).

        :param stroke: A Stroke object.
        :type stroke: 'freestyle.types.Stroke'
        '''
        pass


class BezierCurveShader:
    ''' Class hierarchy: `freestyle.types.StrokeShader` > `BezierCurveShader` [Geometry shader]
    '''

    def __init__(self, error: float = 4.0):
        ''' Builds a BezierCurveShader object.

        :param error: The error we're allowing for the approximation. This error is the max distance allowed between the new curve and the original geometry.
        :type error: float
        '''
        pass

    def shade(self, stroke: 'freestyle.types.Stroke'):
        ''' Transforms the stroke backbone geometry so that it corresponds to a Bezier Curve approximation of the original backbone geometry.

        :param stroke: A Stroke object.
        :type stroke: 'freestyle.types.Stroke'
        '''
        pass


class BlenderTextureShader:
    ''' Class hierarchy: `freestyle.types.StrokeShader` > `BlenderTextureShader` [Texture shader]
    '''

    def __init__(self, texture: typing.Union['bpy.types.LineStyleTextureSlot',
                                             'bpy.types.ShaderNodeTree']):
        ''' Builds a BlenderTextureShader object.

        :param texture: A line style texture slot or a shader node tree to define a set of textures.
        :type texture: typing.Union['bpy.types.LineStyleTextureSlot', 'bpy.types.ShaderNodeTree']
        '''
        pass

    def shade(self, stroke: 'freestyle.types.Stroke'):
        ''' Assigns a blender texture slot to the stroke shading in order to simulate marks.

        :param stroke: A Stroke object.
        :type stroke: 'freestyle.types.Stroke'
        '''
        pass


class CalligraphicShader:
    ''' Class hierarchy: `freestyle.types.StrokeShader` > `CalligraphicShader` [Thickness Shader]
    '''

    def __init__(self, thickness_min: float, thickness_max: float,
                 orientation: typing.
                 Union[typing.Sequence[float], 'mathutils.Vector'],
                 clamp: bool):
        ''' Builds a CalligraphicShader object.

        :param thickness_min: The minimum thickness in the direction perpendicular to the main direction.
        :type thickness_min: float
        :param thickness_max: The maximum thickness in the main direction.
        :type thickness_max: float
        :param orientation: The 2D vector giving the main direction.
        :type orientation: typing.Union[typing.Sequence[float], 'mathutils.Vector']
        :param clamp: If true, the strokes are drawn in black when the stroke direction is between -90 and 90 degrees with respect to the main direction and drawn in white otherwise. If false, the strokes are always drawn in black.
        :type clamp: bool
        '''
        pass

    def shade(self, stroke: 'freestyle.types.Stroke'):
        ''' Assigns thicknesses to the stroke vertices so that the stroke looks like made with a calligraphic tool, i.e. the stroke will be the thickest in a main direction, and the thinnest in the direction perpendicular to this one, and an interpolation in between.

        :param stroke: A Stroke object.
        :type stroke: 'freestyle.types.Stroke'
        '''
        pass


class ColorNoiseShader:
    ''' Class hierarchy: `freestyle.types.StrokeShader` > `ColorNoiseShader` [Color shader]
    '''

    def __init__(self, amplitude: float, period: float):
        ''' Builds a ColorNoiseShader object.

        :param amplitude: The amplitude of the noise signal.
        :type amplitude: float
        :param period: The period of the noise signal.
        :type period: float
        '''
        pass

    def shade(self, stroke: 'freestyle.types.Stroke'):
        ''' Shader to add noise to the stroke colors.

        :param stroke: A Stroke object.
        :type stroke: 'freestyle.types.Stroke'
        '''
        pass


class ConstantColorShader:
    ''' Class hierarchy: `freestyle.types.StrokeShader` > `ConstantColorShader` [Color shader]
    '''

    def __init__(self,
                 red: float,
                 green: float,
                 blue: float,
                 alpha: float = 1.0):
        ''' Builds a ConstantColorShader object.

        :param red: The red component.
        :type red: float
        :param green: The green component.
        :type green: float
        :param blue: The blue component.
        :type blue: float
        :param alpha: The alpha value.
        :type alpha: float
        '''
        pass

    def shade(self, stroke: 'freestyle.types.Stroke'):
        ''' Assigns a constant color to every vertex of the Stroke.

        :param stroke: A Stroke object.
        :type stroke: 'freestyle.types.Stroke'
        '''
        pass


class ConstantThicknessShader:
    ''' Class hierarchy: `freestyle.types.StrokeShader` > `ConstantThicknessShader` [Thickness shader]
    '''

    def __init__(self, thickness: float):
        ''' Builds a ConstantThicknessShader object.

        :param thickness: The thickness that must be assigned to the stroke.
        :type thickness: float
        '''
        pass

    def shade(self, stroke: 'freestyle.types.Stroke'):
        ''' Assigns an absolute constant thickness to every vertex of the Stroke.

        :param stroke: A Stroke object.
        :type stroke: 'freestyle.types.Stroke'
        '''
        pass


class ConstrainedIncreasingThicknessShader:
    ''' Class hierarchy: `freestyle.types.StrokeShader` > `ConstrainedIncreasingThicknessShader` [Thickness shader]
    '''

    def __init__(self, thickness_min: float, thickness_max: float,
                 ratio: float):
        ''' Builds a ConstrainedIncreasingThicknessShader object.

        :param thickness_min: The minimum thickness.
        :type thickness_min: float
        :param thickness_max: The maximum thickness.
        :type thickness_max: float
        :param ratio: The thickness/length ratio that we don't want to exceed.
        :type ratio: float
        '''
        pass

    def shade(self, stroke: 'freestyle.types.Stroke'):
        ''' Same as the `IncreasingThicknessShader`, but here we allow the user to control the thickness/length ratio so that we don't get fat short lines.

        :param stroke: A Stroke object.
        :type stroke: 'freestyle.types.Stroke'
        '''
        pass


class GuidingLinesShader:
    ''' Class hierarchy: `freestyle.types.StrokeShader` > `GuidingLinesShader` [Geometry shader]
    '''

    def __init__(self, offset: float):
        ''' Builds a GuidingLinesShader object.

        :param offset: The line that replaces the stroke is initially in the middle of the initial stroke bounding box. offset is the value of the displacement which is applied to this line along its normal.
        :type offset: float
        '''
        pass

    def shade(self, stroke: 'freestyle.types.Stroke'):
        ''' Shader to modify the Stroke geometry so that it corresponds to its main direction line. This shader must be used together with the splitting operator using the curvature criterion. Indeed, the precision of the approximation will depend on the size of the stroke's pieces. The bigger the pieces are, the rougher the approximation is.

        :param stroke: A Stroke object.
        :type stroke: 'freestyle.types.Stroke'
        '''
        pass


class IncreasingColorShader:
    ''' Class hierarchy: `freestyle.types.StrokeShader` > `IncreasingColorShader` [Color shader]
    '''

    def __init__(self, red_min: float, green_min: float, blue_min: float,
                 alpha_min: float, red_max: float, green_max: float,
                 blue_max: float, alpha_max: float):
        ''' Builds an IncreasingColorShader object.

        :param red_min: The first color red component.
        :type red_min: float
        :param green_min: The first color green component.
        :type green_min: float
        :param blue_min: The first color blue component.
        :type blue_min: float
        :param alpha_min: The first color alpha value.
        :type alpha_min: float
        :param red_max: The second color red component.
        :type red_max: float
        :param green_max: The second color green component.
        :type green_max: float
        :param blue_max: The second color blue component.
        :type blue_max: float
        :param alpha_max: The second color alpha value.
        :type alpha_max: float
        '''
        pass

    def shade(self, stroke: 'freestyle.types.Stroke'):
        ''' Assigns a varying color to the stroke. The user specifies two colors A and B. The stroke color will change linearly from A to B between the first and the last vertex.

        :param stroke: A Stroke object.
        :type stroke: 'freestyle.types.Stroke'
        '''
        pass


class IncreasingThicknessShader:
    ''' Class hierarchy: `freestyle.types.StrokeShader` > `IncreasingThicknessShader` [Thickness shader]
    '''

    def __init__(self, thickness_A: float, thickness_B: float):
        ''' Builds an IncreasingThicknessShader object.

        :param thickness_A: The first thickness value.
        :type thickness_A: float
        :param thickness_B: The second thickness value.
        :type thickness_B: float
        '''
        pass

    def shade(self, stroke: 'freestyle.types.Stroke'):
        ''' Assigns thicknesses values such as the thickness increases from a thickness value A to a thickness value B between the first vertex to the midpoint vertex and then decreases from B to a A between this midpoint vertex and the last vertex. The thickness is linearly interpolated from A to B.

        :param stroke: A Stroke object.
        :type stroke: 'freestyle.types.Stroke'
        '''
        pass


class PolygonalizationShader:
    ''' Class hierarchy: `freestyle.types.StrokeShader` > `PolygonalizationShader` [Geometry shader]
    '''

    def __init__(self, error: float):
        ''' Builds a PolygonalizationShader object.

        :param error: The error we want our polygonal approximation to have with respect to the original geometry. The smaller, the closer the new stroke is to the original one. This error corresponds to the maximum distance between the new stroke and the old one.
        :type error: float
        '''
        pass

    def shade(self, stroke: 'freestyle.types.Stroke'):
        ''' Modifies the Stroke geometry so that it looks more "polygonal". The basic idea is to start from the minimal stroke approximation consisting in a line joining the first vertex to the last one and to subdivide using the original stroke vertices until a certain error is reached.

        :param stroke: A Stroke object.
        :type stroke: 'freestyle.types.Stroke'
        '''
        pass


class RoundCapShader:
    def round_cap_thickness(self, x):
        ''' 

        '''
        pass

    def shade(self, stroke):
        ''' 

        '''
        pass


class SamplingShader:
    ''' Class hierarchy: `freestyle.types.StrokeShader` > `SamplingShader` [Geometry shader]
    '''

    def __init__(self, sampling: float):
        ''' Builds a SamplingShader object.

        :param sampling: The sampling to use for the stroke resampling.
        :type sampling: float
        '''
        pass

    def shade(self, stroke: 'freestyle.types.Stroke'):
        ''' Resamples the stroke.

        :param stroke: A Stroke object.
        :type stroke: 'freestyle.types.Stroke'
        '''
        pass


class SmoothingShader:
    ''' Class hierarchy: `freestyle.types.StrokeShader` > `SmoothingShader` [Geometry shader]
    '''

    def __init__(self,
                 num_iterations: int = 100,
                 factor_point: float = 0.1,
                 factor_curvature: float = 0.0,
                 factor_curvature_difference: float = 0.2,
                 aniso_point: float = 0.0,
                 aniso_normal: float = 0.0,
                 aniso_curvature: float = 0.0,
                 carricature_factor: float = 1.0):
        ''' Builds a SmoothingShader object.

        :param num_iterations: The number of iterations.
        :type num_iterations: int
        :param factor_point: 0.1
        :type factor_point: float
        :param factor_curvature: 0.0
        :type factor_curvature: float
        :param factor_curvature_difference: 0.2
        :type factor_curvature_difference: float
        :param aniso_point: 0.0
        :type aniso_point: float
        :param aniso_normal: 0.0
        :type aniso_normal: float
        :param aniso_curvature: 0.0
        :type aniso_curvature: float
        :param carricature_factor: 1.0
        :type carricature_factor: float
        '''
        pass

    def shade(self, stroke: 'freestyle.types.Stroke'):
        ''' Smooths the stroke by moving the vertices to make the stroke smoother. Uses curvature flow to converge towards a curve of constant curvature. The diffusion method we use is anisotropic to prevent the diffusion across corners.

        :param stroke: A Stroke object.
        :type stroke: 'freestyle.types.Stroke'
        '''
        pass


class SpatialNoiseShader:
    ''' Class hierarchy: `freestyle.types.StrokeShader` > `SpatialNoiseShader` [Geometry shader]
    '''

    def __init__(self, amount: float, scale: float, num_octaves: int,
                 smooth: bool, pure_random: bool):
        ''' Builds a SpatialNoiseShader object.

        :param amount: The amplitude of the noise.
        :type amount: float
        :param scale: The noise frequency.
        :type scale: float
        :param num_octaves: The number of octaves
        :type num_octaves: int
        :param smooth: True if you want the noise to be smooth.
        :type smooth: bool
        :param pure_random: True if you don't want any coherence.
        :type pure_random: bool
        '''
        pass

    def shade(self, stroke: 'freestyle.types.Stroke'):
        ''' Spatial Noise stroke shader. Moves the vertices to make the stroke more noisy.

        :param stroke: A Stroke object.
        :type stroke: 'freestyle.types.Stroke'
        '''
        pass


class SquareCapShader:
    def shade(self, stroke):
        ''' 

        '''
        pass


class StrokeTextureStepShader:
    ''' Class hierarchy: `freestyle.types.StrokeShader` > `StrokeTextureStepShader` [Texture shader]
    '''

    def __init__(self, step: float):
        ''' Builds a StrokeTextureStepShader object.

        :param step: The spacing along the stroke.
        :type step: float
        '''
        pass

    def shade(self, stroke: 'freestyle.types.Stroke'):
        ''' Assigns a spacing factor to the texture coordinates of the Stroke.

        :param stroke: A Stroke object.
        :type stroke: 'freestyle.types.Stroke'
        '''
        pass


class ThicknessNoiseShader:
    ''' Class hierarchy: `freestyle.types.StrokeShader` > `ThicknessNoiseShader` [Thickness shader]
    '''

    def __init__(self, amplitude: float, period: float):
        ''' Builds a ThicknessNoiseShader object.

        :param amplitude: The amplitude of the noise signal.
        :type amplitude: float
        :param period: The period of the noise signal.
        :type period: float
        '''
        pass

    def shade(self, stroke: 'freestyle.types.Stroke'):
        ''' Adds some noise to the stroke thickness.

        :param stroke: A Stroke object.
        :type stroke: 'freestyle.types.Stroke'
        '''
        pass


class TipRemoverShader:
    ''' Class hierarchy: `freestyle.types.StrokeShader` > `TipRemoverShader` [Geometry shader]
    '''

    def __init__(self, tip_length: float):
        ''' Builds a TipRemoverShader object.

        :param tip_length: The length of the piece of stroke we want to remove at each extremity.
        :type tip_length: float
        '''
        pass

    def shade(self, stroke: 'freestyle.types.Stroke'):
        ''' Removes the stroke's extremities.

        :param stroke: A Stroke object.
        :type stroke: 'freestyle.types.Stroke'
        '''
        pass


class py2DCurvatureColorShader:
    ''' Assigns a color (grayscale) to the stroke based on the curvature. A higher curvature will yield a brighter color.
    '''

    def shade(self, stroke):
        ''' 

        '''
        pass


class pyBackboneStretcherNoCuspShader:
    ''' Stretches the stroke's backbone, excluding cusp vertices (end junctions).
    '''

    def shade(self, stroke):
        ''' 

        '''
        pass


class pyBackboneStretcherShader:
    ''' Stretches the stroke's backbone by a given length (in pixels).
    '''

    def shade(self, stroke):
        ''' 

        '''
        pass


class pyBluePrintCirclesShader:
    ''' Draws the silhouette of the object as a circle.
    '''

    def shade(self, stroke):
        ''' 

        '''
        pass


class pyBluePrintDirectedSquaresShader:
    ''' Replaces the stroke with a directed square.
    '''

    def shade(self, stroke):
        ''' 

        '''
        pass


class pyBluePrintEllipsesShader:
    def shade(self, stroke):
        ''' 

        '''
        pass


class pyBluePrintSquaresShader:
    def shade(self, stroke):
        ''' 

        '''
        pass


class pyConstantColorShader:
    ''' Assigns a constant color to the stroke.
    '''

    def shade(self, stroke):
        ''' 

        '''
        pass


class pyConstantThicknessShader:
    ''' Assigns a constant thickness along the stroke.
    '''

    def shade(self, stroke):
        ''' 

        '''
        pass


class pyConstrainedIncreasingThicknessShader:
    ''' Increasingly thickens the stroke, constrained by a ratio of the stroke's length.
    '''

    def shade(self, stroke):
        ''' 

        '''
        pass


class pyDecreasingThicknessShader:
    ''' Inverse of pyIncreasingThicknessShader, decreasingly thickens the stroke.
    '''

    def shade(self, stroke):
        ''' 

        '''
        pass


class pyDepthDiscontinuityThicknessShader:
    ''' Assigns a thickness to the stroke based on the stroke's distance to the camera (Z-value).
    '''

    def shade(self, stroke):
        ''' 

        '''
        pass


class pyDiffusion2Shader:
    ''' Iteratively adds an offset to the position of each stroke vertex in the direction perpendicular to the stroke direction at the point. The offset is scaled by the 2D curvature (i.e. how quickly the stroke curve is) at the point.
    '''

    def shade(self, stroke):
        ''' 

        '''
        pass


class pyFXSVaryingThicknessWithDensityShader:
    ''' Assigns thickness to a stroke based on the density of the diffuse map.
    '''

    def shade(self, stroke):
        ''' 

        '''
        pass


class pyGuidingLineShader:
    def shade(self, stroke):
        ''' 

        '''
        pass


class pyHLRShader:
    ''' Controls visibility based upon the quantitative invisibility (QI) based on hidden line removal (HLR).
    '''

    def shade(self, stroke):
        ''' 

        '''
        pass


class pyImportance2DThicknessShader:
    ''' Assigns thickness based on distance to a given point in 2D space. the thickness is inverted, so the vertices closest to the specified point have the lowest thickness.
    '''

    def shade(self, stroke):
        ''' 

        '''
        pass


class pyImportance3DThicknessShader:
    ''' Assigns thickness based on distance to a given point in 3D space.
    '''

    def shade(self, stroke):
        ''' 

        '''
        pass


class pyIncreasingColorShader:
    ''' Fades from one color to another along the stroke.
    '''

    def shade(self, stroke):
        ''' 

        '''
        pass


class pyIncreasingThicknessShader:
    ''' Increasingly thickens the stroke.
    '''

    def shade(self, stroke):
        ''' 

        '''
        pass


class pyInterpolateColorShader:
    ''' Fades from one color to another and back.
    '''

    def shade(self, stroke):
        ''' 

        '''
        pass


class pyLengthDependingBackboneStretcherShader:
    ''' Stretches the stroke's backbone proportional to the stroke's length NOTE: you'll probably want an l somewhere between (0.5 - 0). A value that is too high may yield unexpected results.
    '''

    def shade(self, stroke):
        ''' 

        '''
        pass


class pyMaterialColorShader:
    ''' Assigns the color of the underlying material to the stroke.
    '''

    def shade(self, stroke):
        ''' 

        '''
        pass


class pyModulateAlphaShader:
    ''' Limits the stroke's alpha between a min and max value.
    '''

    def shade(self, stroke):
        ''' 

        '''
        pass


class pyNonLinearVaryingThicknessShader:
    ''' Assigns thickness to a stroke based on an exponential function.
    '''

    def shade(self, stroke):
        ''' 

        '''
        pass


class pyPerlinNoise1DShader:
    ''' Displaces the stroke using the curvilinear abscissa. This means that lines with the same length and sampling interval will be identically distorted.
    '''

    def shade(self, stroke):
        ''' 

        '''
        pass


class pyPerlinNoise2DShader:
    ''' Displaces the stroke using the strokes coordinates. This means that in a scene no strokes will be distorted identically. More information on the noise shaders can be found at: https://freestyleintegration.wordpress.com/2011/09/25/development-updates-on-september-25/
    '''

    def shade(self, stroke):
        ''' 

        '''
        pass


class pyRandomColorShader:
    ''' Assigns a color to the stroke based on given seed.
    '''

    def shade(self, stroke):
        ''' 

        '''
        pass


class pySLERPThicknessShader:
    ''' Assigns thickness to a stroke based on spherical linear interpolation.
    '''

    def shade(self, stroke):
        ''' 

        '''
        pass


class pySamplingShader:
    ''' Resamples the stroke, which gives the stroke the amount of vertices specified.
    '''

    def shade(self, stroke):
        ''' 

        '''
        pass


class pySinusDisplacementShader:
    ''' Displaces the stroke in the shape of a sine wave.
    '''

    def shade(self, stroke):
        ''' 

        '''
        pass


class pyTVertexRemoverShader:
    ''' Removes t-vertices from the stroke.
    '''

    def shade(self, stroke):
        ''' 

        '''
        pass


class pyTVertexThickenerShader:
    ''' Thickens TVertices (visual intersections between two edges).
    '''

    def shade(self, stroke):
        ''' 

        '''
        pass


class pyTimeColorShader:
    ''' Assigns a grayscale value that increases for every vertex. The brightness will increase along the stroke.
    '''

    def shade(self, stroke):
        ''' 

        '''
        pass


class pyTipRemoverShader:
    ''' Removes the tips of the stroke. Undocumented
    '''

    def shade(self, stroke):
        ''' 

        '''
        pass


class pyZDependingThicknessShader:
    ''' Assigns thickness based on an object's local Z depth (point closest to camera is 1, point furthest from camera is zero).
    '''

    def shade(self, stroke):
        ''' 

        '''
        pass



import sys
import typing

GenericType = typing.TypeVar("GenericType")


def cleanup_po(args, settings):
    ''' 

    '''

    pass


def language_menu(args, settings):
    ''' 

    '''

    pass


def main():
    ''' 

    '''

    pass


def rtl_process_po(args, settings):
    ''' 

    '''

    pass


def strip_po(args, settings):
    ''' 

    '''

    pass


def update_po(args, settings):
    ''' 

    '''

    pass



import sys
import typing

GenericType = typing.TypeVar("GenericType")


def blend_get():
    ''' Current blending equation.

    '''

    pass


def blend_set(mode: str):
    ''' Defines the fixed pipeline blending equation.

    :param mode: The type of blend mode. * ``NONE`` No blending. * ``ALPHA`` The original color channels are interpolated according to the alpha value. * ``ALPHA_PREMULT`` The original color channels are interpolated according to the alpha value with the new colors pre-multiplied by this value. * ``ADDITIVE`` The original color channels are added by the corresponding ones. * ``ADDITIVE_PREMULT`` The original color channels are added by the corresponding ones that are pre-multiplied by the alpha value. * ``MULTIPLY`` The original color channels are multiplied by the corresponding ones. * ``SUBTRACT`` The original color channels are subtracted by the corresponding ones. * ``INVERT`` The original color channels are replaced by its complementary color.
    :type mode: str
    '''

    pass


def clip_distances_set(distances_enabled: int):
    ''' Sets the number of `gl_ClipDistance` planes used for clip geometry.

    :param distances_enabled: Number of clip distances enabled.
    :type distances_enabled: int
    '''

    pass


def color_mask_set(r: bool, g: bool, b: bool, a: bool):
    ''' Enable or disable writing of frame buffer color components.

    :param r: components red, green, blue, and alpha.
    :type r: bool
    :param g: components red, green, blue, and alpha.
    :type g: bool
    :param b: components red, green, blue, and alpha.
    :type b: bool
    :param a: components red, green, blue, and alpha.
    :type a: bool
    '''

    pass


def depth_mask_get():
    ''' Writing status in the depth component.

    '''

    pass


def depth_mask_set(value: typing.Any):
    ''' Write to depth component.

    :type near: bool
    :param value:  True for writing to the depth component.
    :type value: typing.Any
    '''

    pass


def depth_test_get():
    ''' Current depth_test equation.

    '''

    pass


def depth_test_set(mode: str):
    ''' Defines the depth_test equation.

    :param mode: The depth test equation name. Possible values are `NONE`, `ALWAYS`, `LESS`, `LESS_EQUAL`, `EQUAL`, `GREATER` and `GREATER_EQUAL`.
    :type mode: str
    '''

    pass


def face_culling_set(culling):
    ''' Specify whether none, front-facing or back-facing facets can be culled.

    :param mode: `NONE`, `FRONT` or `BACK`.
    :type mode: str
    '''

    pass


def framebuffer_active_get(enable):
    ''' Return the active frame-buffer in context.

    '''

    pass


def front_facing_set(invert: typing.Any):
    ''' Specifies the orientation of front-facing polygons.

    :type mode: bool
    :param invert:  True for clockwise polygons as front-facing.
    :type invert: typing.Any
    '''

    pass


def line_width_get():
    ''' Current width of rasterized lines.

    '''

    pass


def line_width_set(width):
    ''' Specify the width of rasterized lines.

    :type mode: float
    :param size:  New width.
    :type size: typing.Any
    '''

    pass


def point_size_set(size: typing.Any):
    ''' Specify the diameter of rasterized points.

    :type mode: float
    :param size:  New diameter.
    :type size: typing.Any
    '''

    pass


def program_point_size_set(enable: bool):
    ''' If enabled, the derived point size is taken from the (potentially clipped) shader builtin gl_PointSize.

    :param enable: True for shader builtin gl_PointSize.
    :type enable: bool
    '''

    pass


def scissor_get() -> typing.Union[int, int]:
    ''' Retrieve the scissors of the active framebuffer. Note: Only valid between 'scissor_set' and a framebuffer rebind.

    :rtype: typing.Union[int, int]
    :return: The scissor of the active framebuffer as a tuple (x, y, xsize, ysize). x, y: lower left corner of the scissor rectangle, in pixels. xsize, ysize: width and height of the scissor rectangle.
    '''

    pass


def scissor_set(x: int, y: int, xsize: int, ysize: int):
    ''' Specifies the scissor area of the active framebuffer. Note: The scissor state is not saved upon framebuffer rebind.

    :param x: lower left corner of the scissor rectangle, in pixels.
    :type x: int
    :param y: lower left corner of the scissor rectangle, in pixels.
    :type y: int
    :param xsize: width and height of the scissor rectangle.
    :type xsize: int
    :param ysize: width and height of the scissor rectangle.
    :type ysize: int
    '''

    pass


def scissor_test_set(enable: bool):
    ''' Enable/disable scissor testing on the active framebuffer.

    :param enable: True - enable scissor testing. False - disable scissor testing.
    :type enable: bool
    '''

    pass


def viewport_get():
    ''' Viewport of the active framebuffer.

    '''

    pass


def viewport_set(x: int, y: int, xsize: int, ysize: int):
    ''' Specifies the viewport of the active framebuffer. Note: The viewport state is not saved upon framebuffer rebind.

    :param x: lower left corner of the viewport_set rectangle, in pixels.
    :type x: int
    :param y: lower left corner of the viewport_set rectangle, in pixels.
    :type y: int
    :param xsize: width and height of the viewport_set.
    :type xsize: int
    :param ysize: width and height of the viewport_set.
    :type ysize: int
    '''

    pass



import sys
import typing
import mathutils

GenericType = typing.TypeVar("GenericType")


def cell(position: typing.Union[typing.Sequence[float], 'mathutils.Vector']
         ) -> float:
    ''' Returns cell noise value at the specified position.

    :param position: The position to evaluate the selected noise function.
    :type position: typing.Union[typing.Sequence[float], 'mathutils.Vector']
    :rtype: float
    :return: The cell noise value.
    '''

    pass


def cell_vector(
        position: typing.Union[typing.Sequence[float], 'mathutils.Vector']
) -> 'mathutils.Vector':
    ''' Returns cell noise vector at the specified position.

    :param position: The position to evaluate the selected noise function.
    :type position: typing.Union[typing.Sequence[float], 'mathutils.Vector']
    :rtype: 'mathutils.Vector'
    :return: The cell noise vector.
    '''

    pass


def fractal(position: typing.Union[typing.Sequence[float], 'mathutils.Vector'],
            H: float,
            lacunarity: float,
            octaves: int,
            noise_basis: str = 'PERLIN_ORIGINAL') -> float:
    ''' Returns the fractal Brownian motion (fBm) noise value from the noise basis at the specified position.

    :param position: The position to evaluate the selected noise function.
    :type position: typing.Union[typing.Sequence[float], 'mathutils.Vector']
    :param H: The fractal increment factor.
    :type H: float
    :param lacunarity: The gap between successive frequencies.
    :type lacunarity: float
    :param octaves: The number of different noise frequencies used.
    :type octaves: int
    :param noise_basis: Enumerator in ['BLENDER', 'PERLIN_ORIGINAL', 'PERLIN_NEW', 'VORONOI_F1', 'VORONOI_F2', 'VORONOI_F3', 'VORONOI_F4', 'VORONOI_F2F1', 'VORONOI_CRACKLE', 'CELLNOISE'].
    :type noise_basis: str
    :rtype: float
    :return: The fractal Brownian motion noise value.
    '''

    pass


def hetero_terrain(
        position: typing.Union[typing.Sequence[float], 'mathutils.Vector'],
        H: float,
        lacunarity: float,
        octaves: int,
        offset: float,
        noise_basis: str = 'PERLIN_ORIGINAL') -> float:
    ''' Returns the heterogeneous terrain value from the noise basis at the specified position.

    :param position: The position to evaluate the selected noise function.
    :type position: typing.Union[typing.Sequence[float], 'mathutils.Vector']
    :param H: The fractal dimension of the roughest areas.
    :type H: float
    :param lacunarity: The gap between successive frequencies.
    :type lacunarity: float
    :param octaves: The number of different noise frequencies used.
    :type octaves: int
    :param offset: The height of the terrain above 'sea level'.
    :type offset: float
    :param noise_basis: Enumerator in ['BLENDER', 'PERLIN_ORIGINAL', 'PERLIN_NEW', 'VORONOI_F1', 'VORONOI_F2', 'VORONOI_F3', 'VORONOI_F4', 'VORONOI_F2F1', 'VORONOI_CRACKLE', 'CELLNOISE'].
    :type noise_basis: str
    :rtype: float
    :return: The heterogeneous terrain value.
    '''

    pass


def hybrid_multi_fractal(
        position: typing.Union[typing.Sequence[float], 'mathutils.Vector'],
        H: float,
        lacunarity: float,
        octaves: int,
        offset: float,
        gain: float,
        noise_basis: str = 'PERLIN_ORIGINAL') -> float:
    ''' Returns hybrid multifractal value from the noise basis at the specified position.

    :param position: The position to evaluate the selected noise function.
    :type position: typing.Union[typing.Sequence[float], 'mathutils.Vector']
    :param H: The fractal dimension of the roughest areas.
    :type H: float
    :param lacunarity: The gap between successive frequencies.
    :type lacunarity: float
    :param octaves: The number of different noise frequencies used.
    :type octaves: int
    :param offset: The height of the terrain above 'sea level'.
    :type offset: float
    :param gain: Scaling applied to the values.
    :type gain: float
    :param noise_basis: Enumerator in ['BLENDER', 'PERLIN_ORIGINAL', 'PERLIN_NEW', 'VORONOI_F1', 'VORONOI_F2', 'VORONOI_F3', 'VORONOI_F4', 'VORONOI_F2F1', 'VORONOI_CRACKLE', 'CELLNOISE'].
    :type noise_basis: str
    :rtype: float
    :return: The hybrid multifractal value.
    '''

    pass


def multi_fractal(
        position: typing.Union[typing.Sequence[float], 'mathutils.Vector'],
        H: float,
        lacunarity: float,
        octaves: int,
        noise_basis: str = 'PERLIN_ORIGINAL') -> float:
    ''' Returns multifractal noise value from the noise basis at the specified position.

    :param position: The position to evaluate the selected noise function.
    :type position: typing.Union[typing.Sequence[float], 'mathutils.Vector']
    :param H: The fractal increment factor.
    :type H: float
    :param lacunarity: The gap between successive frequencies.
    :type lacunarity: float
    :param octaves: The number of different noise frequencies used.
    :type octaves: int
    :param noise_basis: Enumerator in ['BLENDER', 'PERLIN_ORIGINAL', 'PERLIN_NEW', 'VORONOI_F1', 'VORONOI_F2', 'VORONOI_F3', 'VORONOI_F4', 'VORONOI_F2F1', 'VORONOI_CRACKLE', 'CELLNOISE'].
    :type noise_basis: str
    :rtype: float
    :return: The multifractal noise value.
    '''

    pass


def noise(position: typing.Union[typing.Sequence[float], 'mathutils.Vector'],
          noise_basis: str = 'PERLIN_ORIGINAL') -> float:
    ''' Returns noise value from the noise basis at the position specified.

    :param position: The position to evaluate the selected noise function.
    :type position: typing.Union[typing.Sequence[float], 'mathutils.Vector']
    :param noise_basis: Enumerator in ['BLENDER', 'PERLIN_ORIGINAL', 'PERLIN_NEW', 'VORONOI_F1', 'VORONOI_F2', 'VORONOI_F3', 'VORONOI_F4', 'VORONOI_F2F1', 'VORONOI_CRACKLE', 'CELLNOISE'].
    :type noise_basis: str
    :rtype: float
    :return: The noise value.
    '''

    pass


def noise_vector(
        position: typing.Union[typing.Sequence[float], 'mathutils.Vector'],
        noise_basis: str = 'PERLIN_ORIGINAL') -> 'mathutils.Vector':
    ''' Returns the noise vector from the noise basis at the specified position.

    :param position: The position to evaluate the selected noise function.
    :type position: typing.Union[typing.Sequence[float], 'mathutils.Vector']
    :param noise_basis: Enumerator in ['BLENDER', 'PERLIN_ORIGINAL', 'PERLIN_NEW', 'VORONOI_F1', 'VORONOI_F2', 'VORONOI_F3', 'VORONOI_F4', 'VORONOI_F2F1', 'VORONOI_CRACKLE', 'CELLNOISE'].
    :type noise_basis: str
    :rtype: 'mathutils.Vector'
    :return: The noise vector.
    '''

    pass


def random() -> float:
    ''' Returns a random number in the range [0, 1).

    :rtype: float
    :return: The random number.
    '''

    pass


def random_unit_vector(size: int = 3) -> 'mathutils.Vector':
    ''' Returns a unit vector with random entries.

    :param size: The size of the vector to be produced, in the range [2, 4].
    :type size: int
    :rtype: 'mathutils.Vector'
    :return: The random unit vector.
    '''

    pass


def random_vector(size: int = 3) -> 'mathutils.Vector':
    ''' Returns a vector with random entries in the range (-1, 1).

    :param size: The size of the vector to be produced.
    :type size: int
    :rtype: 'mathutils.Vector'
    :return: The random vector.
    '''

    pass


def ridged_multi_fractal(
        position: typing.Union[typing.Sequence[float], 'mathutils.Vector'],
        H: float,
        lacunarity: float,
        octaves: int,
        offset: float,
        gain: float,
        noise_basis: str = 'PERLIN_ORIGINAL') -> float:
    ''' Returns ridged multifractal value from the noise basis at the specified position.

    :param position: The position to evaluate the selected noise function.
    :type position: typing.Union[typing.Sequence[float], 'mathutils.Vector']
    :param H: The fractal dimension of the roughest areas.
    :type H: float
    :param lacunarity: The gap between successive frequencies.
    :type lacunarity: float
    :param octaves: The number of different noise frequencies used.
    :type octaves: int
    :param offset: The height of the terrain above 'sea level'.
    :type offset: float
    :param gain: Scaling applied to the values.
    :type gain: float
    :param noise_basis: Enumerator in ['BLENDER', 'PERLIN_ORIGINAL', 'PERLIN_NEW', 'VORONOI_F1', 'VORONOI_F2', 'VORONOI_F3', 'VORONOI_F4', 'VORONOI_F2F1', 'VORONOI_CRACKLE', 'CELLNOISE'].
    :type noise_basis: str
    :rtype: float
    :return: The ridged multifractal value.
    '''

    pass


def seed_set(seed: int):
    ''' Sets the random seed used for random_unit_vector, and random.

    :param seed: Seed used for the random generator. When seed is zero, the current time will be used instead.
    :type seed: int
    '''

    pass


def turbulence(
        position: typing.Union[typing.Sequence[float], 'mathutils.Vector'],
        octaves: int,
        hard: bool,
        noise_basis: str = 'PERLIN_ORIGINAL',
        amplitude_scale: float = 0.5,
        frequency_scale: float = 2.0) -> float:
    ''' Returns the turbulence value from the noise basis at the specified position.

    :param position: The position to evaluate the selected noise function.
    :type position: typing.Union[typing.Sequence[float], 'mathutils.Vector']
    :param octaves: The number of different noise frequencies used.
    :type octaves: int
    :param hard: Specifies whether returned turbulence is hard (sharp transitions) or soft (smooth transitions).
    :type hard: bool
    :param noise_basis: Enumerator in ['BLENDER', 'PERLIN_ORIGINAL', 'PERLIN_NEW', 'VORONOI_F1', 'VORONOI_F2', 'VORONOI_F3', 'VORONOI_F4', 'VORONOI_F2F1', 'VORONOI_CRACKLE', 'CELLNOISE'].
    :type noise_basis: str
    :param amplitude_scale: The amplitude scaling factor.
    :type amplitude_scale: float
    :param frequency_scale: The frequency scaling factor
    :type frequency_scale: float
    :rtype: float
    :return: The turbulence value.
    '''

    pass


def turbulence_vector(
        position: typing.Union[typing.Sequence[float], 'mathutils.Vector'],
        octaves: int,
        hard: bool,
        noise_basis: str = 'PERLIN_ORIGINAL',
        amplitude_scale: float = 0.5,
        frequency_scale: float = 2.0) -> 'mathutils.Vector':
    ''' Returns the turbulence vector from the noise basis at the specified position.

    :param position: The position to evaluate the selected noise function.
    :type position: typing.Union[typing.Sequence[float], 'mathutils.Vector']
    :param octaves: The number of different noise frequencies used.
    :type octaves: int
    :param hard: Specifies whether returned turbulence is hard (sharp transitions) or soft (smooth transitions).
    :type hard: bool
    :param noise_basis: Enumerator in ['BLENDER', 'PERLIN_ORIGINAL', 'PERLIN_NEW', 'VORONOI_F1', 'VORONOI_F2', 'VORONOI_F3', 'VORONOI_F4', 'VORONOI_F2F1', 'VORONOI_CRACKLE', 'CELLNOISE'].
    :type noise_basis: str
    :param amplitude_scale: The amplitude scaling factor.
    :type amplitude_scale: float
    :param frequency_scale: The frequency scaling factor
    :type frequency_scale: float
    :rtype: 'mathutils.Vector'
    :return: The turbulence vector.
    '''

    pass


def variable_lacunarity(
        position: typing.Union[typing.Sequence[float], 'mathutils.Vector'],
        distortion: float,
        noise_type1: str = 'PERLIN_ORIGINAL',
        noise_type2: str = 'PERLIN_ORIGINAL') -> float:
    ''' Returns variable lacunarity noise value, a distorted variety of noise, from noise type 1 distorted by noise type 2 at the specified position.

    :param position: The position to evaluate the selected noise function.
    :type position: typing.Union[typing.Sequence[float], 'mathutils.Vector']
    :param distortion: The amount of distortion.
    :type distortion: float
    :param noise_type1: Enumerator in ['BLENDER', 'PERLIN_ORIGINAL', 'PERLIN_NEW', 'VORONOI_F1', 'VORONOI_F2', 'VORONOI_F3', 'VORONOI_F4', 'VORONOI_F2F1', 'VORONOI_CRACKLE', 'CELLNOISE'].
    :type noise_type1: str
    :param noise_type2: Enumerator in ['BLENDER', 'PERLIN_ORIGINAL', 'PERLIN_NEW', 'VORONOI_F1', 'VORONOI_F2', 'VORONOI_F3', 'VORONOI_F4', 'VORONOI_F2F1', 'VORONOI_CRACKLE', 'CELLNOISE'].
    :type noise_type2: str
    :rtype: float
    :return: The variable lacunarity noise value.
    '''

    pass


def voronoi(position: typing.Union[typing.Sequence[float], 'mathutils.Vector'],
            distance_metric: str = 'DISTANCE',
            exponent: float = 2.5) -> typing.List['mathutils.Vector']:
    ''' Returns a list of distances to the four closest features and their locations.

    :param position: The position to evaluate the selected noise function.
    :type position: typing.Union[typing.Sequence[float], 'mathutils.Vector']
    :param distance_metric: Enumerator in ['DISTANCE', 'DISTANCE_SQUARED', 'MANHATTAN', 'CHEBYCHEV', 'MINKOVSKY', 'MINKOVSKY_HALF', 'MINKOVSKY_FOUR'].
    :type distance_metric: str
    :param exponent: The exponent for Minkowski distance metric.
    :type exponent: float
    :rtype: typing.List['mathutils.Vector']
    :return: A list of distances to the four closest features and their locations.
    '''

    pass



import sys
import typing
import bpy.types

GenericType = typing.TypeVar("GenericType")


def gizmo_select(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None,
        *,
        extend: typing.Optional[typing.Union[bool, typing.Any]] = False,
        deselect: typing.Optional[typing.Union[bool, typing.Any]] = False,
        toggle: typing.Optional[typing.Union[bool, typing.Any]] = False,
        deselect_all: typing.Optional[typing.Union[bool, typing.Any]] = False,
        select_passthrough: typing.Optional[typing.Union[bool, typing.
                                                         Any]] = False):
    ''' Select the currently highlighted gizmo

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    :param extend: Extend, Extend selection instead of deselecting everything first
    :type extend: typing.Optional[typing.Union[bool, typing.Any]]
    :param deselect: Deselect, Remove from selection
    :type deselect: typing.Optional[typing.Union[bool, typing.Any]]
    :param toggle: Toggle Selection, Toggle the selection
    :type toggle: typing.Optional[typing.Union[bool, typing.Any]]
    :param deselect_all: Deselect On Nothing, Deselect all when nothing under the cursor
    :type deselect_all: typing.Optional[typing.Union[bool, typing.Any]]
    :param select_passthrough: Only Select Unselected, Ignore the select action when the element is already selected
    :type select_passthrough: typing.Optional[typing.Union[bool, typing.Any]]
    '''

    pass


def gizmo_tweak(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None):
    ''' Tweak the active gizmo

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    '''

    pass



import sys
import typing
import bpy.types
import bpy.ops.transform

GenericType = typing.TypeVar("GenericType")


def delete_metaelems(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None,
        *,
        confirm: typing.Optional[typing.Union[bool, typing.Any]] = True):
    ''' Delete selected metaball element(s)

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    :param confirm: Confirm, Prompt for confirmation
    :type confirm: typing.Optional[typing.Union[bool, typing.Any]]
    '''

    pass


def duplicate_metaelems(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None):
    ''' Duplicate selected metaball element(s)

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    '''

    pass


def duplicate_move(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None,
        *,
        MBALL_OT_duplicate_metaelems: typing.
        Optional['duplicate_metaelems'] = None,
        TRANSFORM_OT_translate: typing.
        Optional['bpy.ops.transform.translate'] = None):
    ''' Make copies of the selected metaball elements and move them

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    :param MBALL_OT_duplicate_metaelems: Duplicate Metaball Elements, Duplicate selected metaball element(s)
    :type MBALL_OT_duplicate_metaelems: typing.Optional['duplicate_metaelems']
    :param TRANSFORM_OT_translate: Move, Move selected items
    :type TRANSFORM_OT_translate: typing.Optional['bpy.ops.transform.translate']
    '''

    pass


def hide_metaelems(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None,
        *,
        unselected: typing.Optional[typing.Union[bool, typing.Any]] = False):
    ''' Hide (un)selected metaball element(s)

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    :param unselected: Unselected, Hide unselected rather than selected
    :type unselected: typing.Optional[typing.Union[bool, typing.Any]]
    '''

    pass


def reveal_metaelems(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None,
        *,
        select: typing.Optional[typing.Union[bool, typing.Any]] = True):
    ''' Reveal all hidden metaball elements

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    :param select: Select
    :type select: typing.Optional[typing.Union[bool, typing.Any]]
    '''

    pass


def select_all(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None,
        *,
        action: typing.Optional[typing.Any] = 'TOGGLE'):
    ''' Change selection of all metaball elements

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    :param action: Action, Selection action to execute * ``TOGGLE`` Toggle -- Toggle selection for all elements. * ``SELECT`` Select -- Select all elements. * ``DESELECT`` Deselect -- Deselect all elements. * ``INVERT`` Invert -- Invert selection of all elements.
    :type action: typing.Optional[typing.Any]
    '''

    pass


def select_random_metaelems(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None,
        *,
        ratio: typing.Optional[typing.Any] = 0.5,
        seed: typing.Optional[typing.Any] = 0,
        action: typing.Optional[typing.Any] = 'SELECT'):
    ''' Randomly select metaball elements

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    :param ratio: Ratio, Portion of items to select randomly
    :type ratio: typing.Optional[typing.Any]
    :param seed: Random Seed, Seed for the random number generator
    :type seed: typing.Optional[typing.Any]
    :param action: Action, Selection action to execute * ``SELECT`` Select -- Select all elements. * ``DESELECT`` Deselect -- Deselect all elements.
    :type action: typing.Optional[typing.Any]
    '''

    pass


def select_similar(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None,
        *,
        type: typing.Optional[typing.Any] = 'TYPE',
        threshold: typing.Optional[typing.Any] = 0.1):
    ''' Select similar metaballs by property types

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    :param type: Type
    :type type: typing.Optional[typing.Any]
    :param threshold: Threshold
    :type threshold: typing.Optional[typing.Any]
    '''

    pass



import sys
import typing
import bpy.types

GenericType = typing.TypeVar("GenericType")


def brush_edit(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None,
        *,
        stroke: typing.Optional[bpy.types.bpy_prop_collection[
            'bpy.types.OperatorStrokeElement']] = None):
    ''' Apply a stroke of brush to the particles

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    :param stroke: Stroke
    :type stroke: typing.Optional[bpy.types.bpy_prop_collection['bpy.types.OperatorStrokeElement']]
    '''

    pass


def connect_hair(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None,
        *,
        all: typing.Optional[typing.Union[bool, typing.Any]] = False):
    ''' Connect hair to the emitter mesh

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    :param all: All Hair, Connect all hair systems to the emitter mesh
    :type all: typing.Optional[typing.Union[bool, typing.Any]]
    '''

    pass


def copy_particle_systems(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None,
        *,
        space: typing.Optional[typing.Any] = 'OBJECT',
        remove_target_particles: typing.Optional[typing.Union[bool, typing.
                                                              Any]] = True,
        use_active: typing.Optional[typing.Union[bool, typing.Any]] = False):
    ''' Copy particle systems from the active object to selected objects

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    :param space: Space, Space transform for copying from one object to another * ``OBJECT`` Object -- Copy inside each object's local space. * ``WORLD`` World -- Copy in world space.
    :type space: typing.Optional[typing.Any]
    :param remove_target_particles: Remove Target Particles, Remove particle systems on the target objects
    :type remove_target_particles: typing.Optional[typing.Union[bool, typing.Any]]
    :param use_active: Use Active, Use the active particle system from the context
    :type use_active: typing.Optional[typing.Union[bool, typing.Any]]
    '''

    pass


def delete(override_context: typing.Optional[
        typing.Union[typing.Dict, 'bpy.types.Context']] = None,
           execution_context: typing.Optional[typing.Union[str, int]] = None,
           undo: typing.Optional[bool] = None,
           *,
           type: typing.Optional[typing.Any] = 'PARTICLE'):
    ''' Delete selected particles or keys

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    :param type: Type, Delete a full particle or only keys
    :type type: typing.Optional[typing.Any]
    '''

    pass


def disconnect_hair(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None,
        *,
        all: typing.Optional[typing.Union[bool, typing.Any]] = False):
    ''' Disconnect hair from the emitter mesh

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    :param all: All Hair, Disconnect all hair systems from the emitter mesh
    :type all: typing.Optional[typing.Union[bool, typing.Any]]
    '''

    pass


def duplicate_particle_system(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None,
        *,
        use_duplicate_settings: typing.Optional[typing.Union[bool, typing.
                                                             Any]] = False):
    ''' Duplicate particle system within the active object

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    :param use_duplicate_settings: Duplicate Settings, Duplicate settings as well, so the new particle system uses its own settings
    :type use_duplicate_settings: typing.Optional[typing.Union[bool, typing.Any]]
    '''

    pass


def dupliob_copy(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None):
    ''' Duplicate the current instance object

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    '''

    pass


def dupliob_move_down(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None):
    ''' Move instance object down in the list

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    '''

    pass


def dupliob_move_up(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None):
    ''' Move instance object up in the list

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    '''

    pass


def dupliob_refresh(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None):
    ''' Refresh list of instance objects and their weights

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    '''

    pass


def dupliob_remove(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None):
    ''' Remove the selected instance object

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    '''

    pass


def edited_clear(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None):
    ''' Undo all edition performed on the particle system

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    '''

    pass


def hair_dynamics_preset_add(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None,
        *,
        name: typing.Union[str, typing.Any] = "",
        remove_name: typing.Optional[typing.Union[bool, typing.Any]] = False,
        remove_active: typing.Optional[typing.Union[bool, typing.
                                                    Any]] = False):
    ''' Add or remove a Hair Dynamics Preset :File: `startup/bl_operators/presets.py\:75 <https://projects.blender.org/blender/blender/src/branch/main/scripts/startup/bl_operators/presets.py#L75>`__

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    :param name: Name, Name of the preset, used to make the path name
    :type name: typing.Union[str, typing.Any]
    :param remove_name: remove_name
    :type remove_name: typing.Optional[typing.Union[bool, typing.Any]]
    :param remove_active: remove_active
    :type remove_active: typing.Optional[typing.Union[bool, typing.Any]]
    '''

    pass


def hide(override_context: typing.Optional[
        typing.Union[typing.Dict, 'bpy.types.Context']] = None,
         execution_context: typing.Optional[typing.Union[str, int]] = None,
         undo: typing.Optional[bool] = None,
         *,
         unselected: typing.Optional[typing.Union[bool, typing.Any]] = False):
    ''' Hide selected particles

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    :param unselected: Unselected, Hide unselected rather than selected
    :type unselected: typing.Optional[typing.Union[bool, typing.Any]]
    '''

    pass


def mirror(override_context: typing.Optional[
        typing.Union[typing.Dict, 'bpy.types.Context']] = None,
           execution_context: typing.Optional[typing.Union[str, int]] = None,
           undo: typing.Optional[bool] = None):
    ''' Duplicate and mirror the selected particles along the local X axis

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    '''

    pass


def new(override_context: typing.Optional[
        typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None):
    ''' Add new particle settings

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    '''

    pass


def new_target(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None):
    ''' Add a new particle target

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    '''

    pass


def particle_edit_toggle(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None):
    ''' Toggle particle edit mode

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    '''

    pass


def rekey(override_context: typing.Optional[
        typing.Union[typing.Dict, 'bpy.types.Context']] = None,
          execution_context: typing.Optional[typing.Union[str, int]] = None,
          undo: typing.Optional[bool] = None,
          *,
          keys_number: typing.Optional[typing.Any] = 2):
    ''' Change the number of keys of selected particles (root and tip keys included)

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    :param keys_number: Number of Keys
    :type keys_number: typing.Optional[typing.Any]
    '''

    pass


def remove_doubles(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None,
        *,
        threshold: typing.Optional[typing.Any] = 0.0002):
    ''' Remove selected particles close enough of others

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    :param threshold: Merge Distance, Threshold distance within which particles are removed
    :type threshold: typing.Optional[typing.Any]
    '''

    pass


def reveal(override_context: typing.Optional[
        typing.Union[typing.Dict, 'bpy.types.Context']] = None,
           execution_context: typing.Optional[typing.Union[str, int]] = None,
           undo: typing.Optional[bool] = None,
           *,
           select: typing.Optional[typing.Union[bool, typing.Any]] = True):
    ''' Show hidden particles

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    :param select: Select
    :type select: typing.Optional[typing.Union[bool, typing.Any]]
    '''

    pass


def select_all(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None,
        *,
        action: typing.Optional[typing.Any] = 'TOGGLE'):
    ''' (De)select all particles' keys

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    :param action: Action, Selection action to execute * ``TOGGLE`` Toggle -- Toggle selection for all elements. * ``SELECT`` Select -- Select all elements. * ``DESELECT`` Deselect -- Deselect all elements. * ``INVERT`` Invert -- Invert selection of all elements.
    :type action: typing.Optional[typing.Any]
    '''

    pass


def select_less(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None):
    ''' Deselect boundary selected keys of each particle

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    '''

    pass


def select_linked(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None):
    ''' Select all keys linked to already selected ones

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    '''

    pass


def select_linked_pick(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None,
        *,
        deselect: typing.Optional[typing.Union[bool, typing.Any]] = False,
        location: typing.Optional[typing.Any] = (0, 0)):
    ''' Select nearest particle from mouse pointer

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    :param deselect: Deselect, Deselect linked keys rather than selecting them
    :type deselect: typing.Optional[typing.Union[bool, typing.Any]]
    :param location: Location
    :type location: typing.Optional[typing.Any]
    '''

    pass


def select_more(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None):
    ''' Select keys linked to boundary selected keys of each particle

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    '''

    pass


def select_random(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None,
        *,
        ratio: typing.Optional[typing.Any] = 0.5,
        seed: typing.Optional[typing.Any] = 0,
        action: typing.Optional[typing.Any] = 'SELECT',
        type: typing.Optional[typing.Any] = 'HAIR'):
    ''' Select a randomly distributed set of hair or points

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    :param ratio: Ratio, Portion of items to select randomly
    :type ratio: typing.Optional[typing.Any]
    :param seed: Random Seed, Seed for the random number generator
    :type seed: typing.Optional[typing.Any]
    :param action: Action, Selection action to execute * ``SELECT`` Select -- Select all elements. * ``DESELECT`` Deselect -- Deselect all elements.
    :type action: typing.Optional[typing.Any]
    :param type: Type, Select either hair or points
    :type type: typing.Optional[typing.Any]
    '''

    pass


def select_roots(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None,
        *,
        action: typing.Optional[typing.Any] = 'SELECT'):
    ''' Select roots of all visible particles

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    :param action: Action, Selection action to execute * ``TOGGLE`` Toggle -- Toggle selection for all elements. * ``SELECT`` Select -- Select all elements. * ``DESELECT`` Deselect -- Deselect all elements. * ``INVERT`` Invert -- Invert selection of all elements.
    :type action: typing.Optional[typing.Any]
    '''

    pass


def select_tips(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None,
        *,
        action: typing.Optional[typing.Any] = 'SELECT'):
    ''' Select tips of all visible particles

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    :param action: Action, Selection action to execute * ``TOGGLE`` Toggle -- Toggle selection for all elements. * ``SELECT`` Select -- Select all elements. * ``DESELECT`` Deselect -- Deselect all elements. * ``INVERT`` Invert -- Invert selection of all elements.
    :type action: typing.Optional[typing.Any]
    '''

    pass


def shape_cut(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None):
    ''' Cut hair to conform to the set shape object

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    '''

    pass


def subdivide(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None):
    ''' Subdivide selected particles segments (adds keys)

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    '''

    pass


def target_move_down(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None):
    ''' Move particle target down in the list

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    '''

    pass


def target_move_up(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None):
    ''' Move particle target up in the list

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    '''

    pass


def target_remove(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None):
    ''' Remove the selected particle target

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    '''

    pass


def unify_length(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None):
    ''' Make selected hair the same length

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    '''

    pass


def weight_set(
        override_context: typing.Optional[
            typing.Union[typing.Dict, 'bpy.types.Context']] = None,
        execution_context: typing.Optional[typing.Union[str, int]] = None,
        undo: typing.Optional[bool] = None,
        *,
        factor: typing.Optional[typing.Any] = 1.0):
    ''' Set the weight of selected keys

    :type override_context: typing.Optional[typing.Union[typing.Dict, 'bpy.types.Context']]
    :type execution_context: typing.Optional[typing.Union[str, int]]
    :type undo: typing.Optional[bool]
    :param factor: Factor, Interpolation factor between current brush weight, and keys' weights
    :type factor: typing.Optional[typing.Any]
    '''

    pass



import sys
import typing

GenericType = typing.TypeVar("GenericType")


class I18n:
    parsers = None
    ''' '''

    py_file = None
    ''' '''

    writers = None
    ''' '''

    def check_py_module_has_translations(self, src, settings):
        ''' 

        '''
        pass

    def escape(self, do_all):
        ''' 

        '''
        pass

    def parse(self, kind, src, langs):
        ''' 

        '''
        pass

    def parse_from_po(self, src, langs):
        ''' 

        '''
        pass

    def parse_from_py(self, src, langs):
        ''' 

        '''
        pass

    def print_stats(self, prefix, print_msgs):
        ''' 

        '''
        pass

    def unescape(self, do_all):
        ''' 

        '''
        pass

    def update_info(self):
        ''' 

        '''
        pass

    def write(self, kind, langs):
        ''' 

        '''
        pass

    def write_to_po(self, langs):
        ''' 

        '''
        pass

    def write_to_py(self, langs):
        ''' 

        '''
        pass


class I18nMessage:
    comment_lines = None
    ''' '''

    is_commented = None
    ''' '''

    is_fuzzy = None
    ''' '''

    is_tooltip = None
    ''' '''

    msgctxt = None
    ''' '''

    msgctxt_lines = None
    ''' '''

    msgid = None
    ''' '''

    msgid_lines = None
    ''' '''

    msgstr = None
    ''' '''

    msgstr_lines = None
    ''' '''

    settings = None
    ''' '''

    sources = None
    ''' '''

    def copy(self):
        ''' 

        '''
        pass

    def do_escape(self, txt):
        ''' 

        '''
        pass

    def do_unescape(self, txt):
        ''' 

        '''
        pass

    def escape(self, do_all):
        ''' 

        '''
        pass

    def normalize(self, max_len):
        ''' 

        '''
        pass

    def unescape(self, do_all):
        ''' 

        '''
        pass


class I18nMessages:
    parsers = None
    ''' '''

    writers = None
    ''' '''

    def check(self, fix):
        ''' 

        '''
        pass

    def clean_commented(self):
        ''' 

        '''
        pass

    def escape(self, do_all):
        ''' 

        '''
        pass

    def find_best_messages_matches(self, msgs, msgmap, rna_ctxt,
                                   rna_struct_name, rna_prop_name,
                                   rna_enum_name):
        ''' 

        '''
        pass

    def gen_empty_messages(self, uid, blender_ver, blender_hash, time, year,
                           default_copyright, settings):
        ''' 

        '''
        pass

    def invalidate_reverse_cache(self, rebuild_now):
        ''' 

        '''
        pass

    def merge(self, msgs, replace):
        ''' 

        '''
        pass

    def normalize(self, max_len):
        ''' 

        '''
        pass

    def parse(self, kind, key, src):
        ''' 

        '''
        pass

    def parse_messages_from_po(self, src, key):
        ''' 

        '''
        pass

    def print_info(self, prefix, output, print_stats, print_errors):
        ''' 

        '''
        pass

    def rtl_process(self):
        ''' 

        '''
        pass

    def unescape(self, do_all):
        ''' 

        '''
        pass

    def update(self, ref, use_similar, keep_old_commented):
        ''' 

        '''
        pass

    def update_info(self):
        ''' 

        '''
        pass

    def write(self, kind, dest):
        ''' 

        '''
        pass

    def write_messages_to_mo(self, fname):
        ''' 

        '''
        pass

    def write_messages_to_po(self, fname, compact):
        ''' 

        '''
        pass


def enable_addons(addons, support, disable, check_only):
    ''' 

    '''

    pass


def find_best_isocode_matches(uid, iso_codes):
    ''' 

    '''

    pass


def get_best_similar(data):
    ''' 

    '''

    pass


def get_po_files_from_dir(root_dir, langs):
    ''' 

    '''

    pass


def is_valid_po_path(path):
    ''' 

    '''

    pass


def list_po_dir(root_path, settings):
    ''' 

    '''

    pass


def locale_explode(locale):
    ''' 

    '''

    pass


def locale_match(loc1, loc2):
    ''' 

    '''

    pass
